@@ -97,82 +97,83 @@ def main():
 
     parser_env = subparsers.add_parser('env')
     parser_env_sub = parser_env.add_subparsers()
-    parser_env_get = parser_env_sub.add_parser('get')
-    parser_env_get.add_argument('get',
+    parser_env_get = parser_env_sub.add_parser('get',
                         help='get service env.')
+    parser_env_get.add_argument('service')
     parser_env_get.set_defaults(func=_env_get)
 
     parser_env_load = parser_env_sub.add_parser('load',
                         help='load local file into env.')
-    parser_env_load.add_argument('load',
-                        help='lood local file into env.')
+    parser_env_load.add_argument('env_file')
     parser_env_load.set_defaults(func=_env_load)
 
 
-    parser_tf = subparsers.add_parser('tf')
-    parser_tf.add_argument('cmd',
-                        nargs='*',
+    parser_tf = subparsers.add_parser('tf',
+                        help='terraform command, eg: init plan apply destroy')
+    parser_tf.add_argument('cmd', nargs='*',
                         help='terraform command, eg: init plan apply destroy')
     parser_tf.set_defaults(func=_tf_cmd)
 
     parser_ec2 = subparsers.add_parser('ec2')
     parser_ec2_sub = parser_ec2.add_subparsers()
-    parser_ec2_search = parser_ec2_sub.add_parser('search')
-
-    parser_ec2_search.add_argument('search',
+    parser_ec2_search = parser_ec2_sub.add_parser('search',
                         help='search out ec2 instance id by keyword: ip, name')
+    parser_ec2_search.add_argument('ec2')
     parser_ec2_search.set_defaults(func=_ec2_search)
 
-    parser_ec2_login = parser_ec2_sub.add_parser('login')
-    parser_ec2_login.add_argument('login',
+    parser_ec2_login = parser_ec2_sub.add_parser('login',
                         help='ssm login into ec2.')
+    parser_ec2_login.add_argument('ec2')
     parser_ec2_login.set_defaults(func=_ec2_login)
 
 
     parser_sm = subparsers.add_parser('sm')
-    parser_sm_sub = parser_sm.add_subparsers(help='sub func: search, get, create, update')
-    parser_sm_search = parser_sm_sub.add_parser('search')
-    parser_sm_search.add_argument('search',
+    parser_sm_sub = parser_sm.add_subparsers()
+    parser_sm_search = parser_sm_sub.add_parser('search',
                         help='search secrets manager')
+    parser_sm_search.add_argument('secret')
     parser_sm_search.set_defaults(func=_sm_search)
 
-    parser_sm_get = parser_sm_sub.add_parser('get')
-    parser_sm_get.add_argument('get',
+    parser_sm_get = parser_sm_sub.add_parser('get',
                         help='get secrets manager')
+    parser_sm_get.add_argument('secret')
     parser_sm_get.set_defaults(func=_sm_get)
 
-    parser_sm_create = parser_sm_sub.add_parser('create')
-    parser_sm_create.add_argument('create',
-                        action='store_true',
+    parser_sm_create = parser_sm_sub.add_parser('create',
                         help='create secrets manager')
+    parser_sm_create.add_argument('create', action='store_true')
     parser_sm_create.set_defaults(func=_sm_create)
 
-    parser_sm_update = parser_sm_sub.add_parser('update')
-    parser_sm_update.add_argument('update',
+    parser_sm_update = parser_sm_sub.add_parser('update',
                         help='update secrets manager')
+    parser_sm_update.add_argument('secret')
     parser_sm_update.set_defaults(func=_sm_update)
 
-    parser_sm_delete = parser_sm_sub.add_parser('delete')
-    parser_sm_delete.add_argument('delete',
+    parser_sm_delete = parser_sm_sub.add_parser('delete',
                         help='delete secrets manager')
+    parser_sm_delete.add_argument('secret')
     parser_sm_delete.set_defaults(func=_sm_delete)
 
     parser_image = subparsers.add_parser('image')
-    parser_image_sub = parser_image.add_subparsers(help='sub func: list, pull')
-    parser_image_list = parser_image_sub.add_parser('list')
-    parser_image_list.add_argument('list',
-                        help='list service images')
+    parser_image_sub = parser_image.add_subparsers()
+    parser_image_list = parser_image_sub.add_parser('list',
+                        help='list service images.')
+    parser_image_list.add_argument('service')
     parser_image_list.set_defaults(func=_image_list)
 
-    parser_image_pull = parser_image_sub.add_parser('pull')
-    parser_image_pull.add_argument('pull',
-                        help='pull service image')
+    parser_image_pull = parser_image_sub.add_parser('pull',
+                        help='pull service image.')
+    parser_image_pull.add_argument('image',
+                        help='image full name.')
     parser_image_pull.set_defaults(func=_image_pull)
 
-    parser_image_stat = parser_image_sub.add_parser('stat')
-    parser_image_stat.add_argument('stat',
-                        help='showing current image version in specific envs.')
-    parser_image_stat.set_defaults(func=_image_stat)
+    parser_image_run = parser_image_sub.add_parser('run',
+                        help='run service image with envs injected. eg: awsso image run emailserver ac702bf [5051]')
+    parser_image_run.add_argument('service',
+                        help='service name.')
+    parser_image_run.add_argument('tag', nargs='*',
+                        help='service image tag [port]')
+    parser_image_run.set_defaults(func=_image_run)
 
     argcomplete.autocomplete(parser)
     args = parser.parse_args()
@@ -203,31 +204,31 @@ def main():
 #        if args.func == _sso_logout:
 #            _sso_logout()
         if args.func == _env_get:
-            _env_get(args.get)
+            _env_get(args.service)
         if args.func == _env_load:
-            _env_load(args.load)
+            _env_load(args.env_file)
         if args.func == _tf_cmd:
             _tf_cmd(args.cmd)
         if args.func == _ec2_search:
-            _ec2_search(args.search)
+            _ec2_search(args.ec2)
         if args.func == _ec2_login:
-            _ec2_login(args.login)
+            _ec2_login(args.ec2)
         if args.func == _sm_search:
-            _sm_search(args.search)
+            _sm_search(args.secret)
         if args.func == _sm_get:
-            _sm_get(args.get)
+            _sm_get(args.secret)
         if args.func == _sm_create:
             _sm_create()
         if args.func == _sm_update:
-            _sm_update(args.update)
+            _sm_update(args.secret)
         if args.func == _sm_delete:
-            _sm_delete(args.delete)
+            _sm_delete(args.secret)
         if args.func == _image_list:
-            _image_list(args.list)
+            _image_list(args.service)
         if args.func == _image_pull:
-            _image_pull(args.pull)
-        if args.func == _image_stat:
-            _image_stat(args.stat)
+            _image_pull(args.image)
+        if args.func == _image_run:
+            _image_run(args.service, args.tag)
 
 
 def _validate_awscli():
@@ -558,11 +559,14 @@ def _ec2_search(keyword):
             choices=i_list
         ),
     ]
-    answer = inquirer.prompt(questions)
-    instance_id = answer['name'].split(' ')[0]
+    try:
+        answer = inquirer.prompt(questions)
+        instance_id = answer['name'].split(' ')[0]
 
-    print('Will login instance:  {}'.format(answer['name']))
-    os.system('aws ssm start-session --target {} --region {}'.format(instance_id, CURRENT_REGION))
+        print('Will login instance:  {}'.format(answer['name']))
+        os.system('aws ssm start-session --target {} --region {}'.format(instance_id, CURRENT_REGION))
+    except:
+        sys.exit(1)
 
 def _ec2_login(keyword):
     if keyword.startswith('i-'):
@@ -816,7 +820,8 @@ def _image_list(service):
         print('{}.dkr.ecr.{}.amazonaws.com/{}:{}'.format(AWS_OPS1_ACCOUNT, AWS_OPS1_REGION, service, j))
 
 def _image_pull(image):
-    subprocess.run('aws ecr get-login-password --region {1} | docker login --username AWS --password-stdin {0}.dkr.ecr.{1}.amazonaws.com'.format(AWS_OPS1_ACCOUNT, AWS_OPS1_REGION),
+    subprocess.run('aws ecr get-login-password --region {1} | docker login --username AWS \
+        --password-stdin {0}.dkr.ecr.{1}.amazonaws.com'.format(AWS_OPS1_ACCOUNT, AWS_OPS1_REGION),
         shell=True,
         stderr=sys.stderr,
         stdout=sys.stdout,
@@ -826,6 +831,27 @@ def _image_pull(image):
         stdout=sys.stdout,
         check=True)
 
+def _image_run(service, tag):
+    port_cmd = ''
+    if len(tag) == 2:
+        try:
+            if isinstance(int(tag[1]), int) and int(tag[1]) >= 1000:
+                port_cmd = '-p {0}:{0}'.format(tag[1])
+        except:
+            print('service port is invalid.')
+            sys.exit(1)
+        
+    run_cmd = 'docker run --rm -d {4} -v ~/.aws:/root/.aws --env-file ~/.{0}_env \
+{1}.dkr.ecr.{2}.amazonaws.com/{0}:{3}'.format(service, AWS_OPS1_ACCOUNT, AWS_OPS1_REGION, tag[0], port_cmd)
+    try:
+        subprocess.run(run_cmd,
+            shell=True,
+            stderr=sys.stderr,
+            stdout=sys.stdout,
+            check=True)
+    except subprocess.CalledProcessError as e:
+        print(e)
+
 def _image_stat(service):
     pass
 
